/*
  Bevel gear macro by David G. Wilkinson <davidwilkinson@cwcom.net>  11 August 1999
  
                macro parameters;
                
  NtW = number of teeth in gear being defined
  NtP = number of teeth in meshing gear
  DP = diametral pitch. (the number of teeth per unit diameter)
       This is effectively the tooth size. Values from 1 to 20 are common.
       The smaller the DP and the larger the gear.
       e.g. if you want a gear with a pitch diameter of 10 with 40 teeth then DP = 4
       
  FaceWidth = the width of the teeth
  
       The clearance at the bottom of the teeth is also defineable.
       e.g. #declare Clr=0.25;  gives a healthy clearance.
            Values are usually from 0.1 to 0.25.  The default is 0.2
*/
  
#macro BevelGear(NtW,NtP,DP,FaceWidth)
        #ifndef (Clr) #declare Clr=0.2; #end
        #declare ConeAngle= atan2(NtW,NtP); 
        #declare ConeHeight=sqrt((NtW/(2*DP))*(NtW/(2*DP))
                                +(NtP/(2*DP))*(NtP/(2*DP)));
        #declare Dedm = (1+Clr)/DP;  

#local VPRad=NtW/(2*DP)/cos(ConeAngle); // virtual Pitch Radius
#local VBaseRad= VPRad*cos(radians(20)); // from which the involute is generated
#local VNumTeeth = NtW/cos(ConeAngle);  // virtual number of teeth
#local VAngulPitch = 2*pi/VNumTeeth;  // virtual angular pitch in radians
#local InvPoint = array[42]  // array for points on the involute curve
#local Inv20=tan(radians(20))-radians(20); // assumes a pressure angle of 20 degrees
#local Incr=(2+Clr)/(DP*20);  // 20 increments on the involute
#local R1=VPRad-(1+Clr)/DP; // the radius at which the co-ords are calculated

#local Counter = 0;
          #while (Counter<20.5)   // generate the points on the tooth profile
                #local Psi = acos(VBaseRad/R1);
                #local Inv = tan(Psi)-Psi; // the involute function          
                #local InvPoint[Counter] = <R1*sin(Inv-Inv20-VAngulPitch/4),  // left flank
                                            R1*cos(Inv-Inv20-VAngulPitch/4)-VPRad>; 
                #local InvPoint[41-Counter] = <-InvPoint[Counter].x,  // right flank
                                               InvPoint[Counter].y>;                                     
                #local Counter = Counter + 1;
                #local R1 = R1 + Incr;
          #end 
      
        #local Count=0;
        #local Height2=1/(ConeHeight-FaceWidth);
        #declare Tooth = prism { conic_sweep
            1 ,1-FaceWidth/ConeHeight, 43
            #while (Count < 41.5)
                InvPoint[Count]
               #local  Count=Count+1;
            #end
            InvPoint[0] // repeat the first point to close the prism
            scale <1,ConeHeight,1> 
     } 

  #declare Teeth= union{  //  the gear teeth defined
                #local N1 = 0;
                #while (N1<NtW) // place the teeth
                   object {Tooth
                      rotate -x*(180-degrees(ConeAngle))
                      rotate y*N1*360/NtW
                   }
                #local N1=N1+1;
                #end
        }
 
  #local FFPX=(ConeHeight-FaceWidth)*sin(ConeAngle); // calculate blank dimensions
  #local FFPY=(ConeHeight-FaceWidth)*cos(ConeAngle);
  #local FFDedm=Dedm*(ConeHeight-FaceWidth)/ConeHeight;
  
// the following variables can be used to size material 
// added to the back or front of the gear.  "ConeAngle" is also available.
  #declare BackFace=NtP/(2*DP)+((1+Clr)/DP)*sin(ConeAngle);// back face to cone vertex 
  #declare FrontFace=FFPY+FFDedm*sin(ConeAngle); // front face to cone vertex          
  #declare BackRad = (NtW/(2*DP))-Dedm*cos(ConeAngle); // radius of back face
  #declare FrontRad = FFPX-FFDedm*cos(ConeAngle);    // radius of front face (base of teeth)
                
  union { // this is the complete gear       
    object { Teeth }  // instance the teeth
    cone { <0,-(BackFace+.001),0> BackRad // add the conic body
           <0,-FrontFace+.001,0> FrontRad
    }
  }
#end // end of BevelGear macro


